import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { ReviewStatus } from 'app/entities/enumerations/review-status.model';
import { IReviewRating, ReviewRating } from '../review-rating.model';

import { ReviewRatingService } from './review-rating.service';

describe('ReviewRating Service', () => {
  let service: ReviewRatingService;
  let httpMock: HttpTestingController;
  let elemDefault: IReviewRating;
  let expectedResult: IReviewRating | IReviewRating[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(ReviewRatingService);
    httpMock = TestBed.inject(HttpTestingController);

    elemDefault = {
      id: 0,
      comment: 'AAAAAAA',
      status: ReviewStatus.REVIEW_IN_PROGRESS,
    };
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = Object.assign({}, elemDefault);

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(elemDefault);
    });

    it('should create a ReviewRating', () => {
      const returnedFromService = Object.assign(
        {
          id: 0,
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.create(new ReviewRating()).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a ReviewRating', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          comment: 'BBBBBB',
          status: 'BBBBBB',
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.update(expected).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a ReviewRating', () => {
      const patchObject = Object.assign(
        {
          status: 'BBBBBB',
        },
        new ReviewRating()
      );

      const returnedFromService = Object.assign(patchObject, elemDefault);

      const expected = Object.assign({}, returnedFromService);

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of ReviewRating', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          comment: 'BBBBBB',
          status: 'BBBBBB',
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toContainEqual(expected);
    });

    it('should delete a ReviewRating', () => {
      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult);
    });

    describe('addReviewRatingToCollectionIfMissing', () => {
      it('should add a ReviewRating to an empty array', () => {
        const reviewRating: IReviewRating = { id: 123 };
        expectedResult = service.addReviewRatingToCollectionIfMissing([], reviewRating);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(reviewRating);
      });

      it('should not add a ReviewRating to an array that contains it', () => {
        const reviewRating: IReviewRating = { id: 123 };
        const reviewRatingCollection: IReviewRating[] = [
          {
            ...reviewRating,
          },
          { id: 456 },
        ];
        expectedResult = service.addReviewRatingToCollectionIfMissing(reviewRatingCollection, reviewRating);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a ReviewRating to an array that doesn't contain it", () => {
        const reviewRating: IReviewRating = { id: 123 };
        const reviewRatingCollection: IReviewRating[] = [{ id: 456 }];
        expectedResult = service.addReviewRatingToCollectionIfMissing(reviewRatingCollection, reviewRating);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(reviewRating);
      });

      it('should add only unique ReviewRating to an array', () => {
        const reviewRatingArray: IReviewRating[] = [{ id: 123 }, { id: 456 }, { id: 27556 }];
        const reviewRatingCollection: IReviewRating[] = [{ id: 123 }];
        expectedResult = service.addReviewRatingToCollectionIfMissing(reviewRatingCollection, ...reviewRatingArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const reviewRating: IReviewRating = { id: 123 };
        const reviewRating2: IReviewRating = { id: 456 };
        expectedResult = service.addReviewRatingToCollectionIfMissing([], reviewRating, reviewRating2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(reviewRating);
        expect(expectedResult).toContain(reviewRating2);
      });

      it('should accept null and undefined values', () => {
        const reviewRating: IReviewRating = { id: 123 };
        expectedResult = service.addReviewRatingToCollectionIfMissing([], null, reviewRating, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(reviewRating);
      });

      it('should return initial array if no ReviewRating is added', () => {
        const reviewRatingCollection: IReviewRating[] = [{ id: 123 }];
        expectedResult = service.addReviewRatingToCollectionIfMissing(reviewRatingCollection, undefined, null);
        expect(expectedResult).toEqual(reviewRatingCollection);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
