import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { CheckFrequency } from 'app/entities/enumerations/check-frequency.model';
import { IUserWatchList, UserWatchList } from '../user-watch-list.model';

import { UserWatchListService } from './user-watch-list.service';

describe('UserWatchList Service', () => {
  let service: UserWatchListService;
  let httpMock: HttpTestingController;
  let elemDefault: IUserWatchList;
  let expectedResult: IUserWatchList | IUserWatchList[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(UserWatchListService);
    httpMock = TestBed.inject(HttpTestingController);

    elemDefault = {
      id: 0,
      name: 'AAAAAAA',
      checkFrequency: CheckFrequency.NEVER,
    };
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = Object.assign({}, elemDefault);

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(elemDefault);
    });

    it('should create a UserWatchList', () => {
      const returnedFromService = Object.assign(
        {
          id: 0,
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.create(new UserWatchList()).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a UserWatchList', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          name: 'BBBBBB',
          checkFrequency: 'BBBBBB',
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.update(expected).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a UserWatchList', () => {
      const patchObject = Object.assign(
        {
          name: 'BBBBBB',
          checkFrequency: 'BBBBBB',
        },
        new UserWatchList()
      );

      const returnedFromService = Object.assign(patchObject, elemDefault);

      const expected = Object.assign({}, returnedFromService);

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of UserWatchList', () => {
      const returnedFromService = Object.assign(
        {
          id: 1,
          name: 'BBBBBB',
          checkFrequency: 'BBBBBB',
        },
        elemDefault
      );

      const expected = Object.assign({}, returnedFromService);

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toContainEqual(expected);
    });

    it('should delete a UserWatchList', () => {
      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult);
    });

    describe('addUserWatchListToCollectionIfMissing', () => {
      it('should add a UserWatchList to an empty array', () => {
        const userWatchList: IUserWatchList = { id: 123 };
        expectedResult = service.addUserWatchListToCollectionIfMissing([], userWatchList);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(userWatchList);
      });

      it('should not add a UserWatchList to an array that contains it', () => {
        const userWatchList: IUserWatchList = { id: 123 };
        const userWatchListCollection: IUserWatchList[] = [
          {
            ...userWatchList,
          },
          { id: 456 },
        ];
        expectedResult = service.addUserWatchListToCollectionIfMissing(userWatchListCollection, userWatchList);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a UserWatchList to an array that doesn't contain it", () => {
        const userWatchList: IUserWatchList = { id: 123 };
        const userWatchListCollection: IUserWatchList[] = [{ id: 456 }];
        expectedResult = service.addUserWatchListToCollectionIfMissing(userWatchListCollection, userWatchList);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(userWatchList);
      });

      it('should add only unique UserWatchList to an array', () => {
        const userWatchListArray: IUserWatchList[] = [{ id: 123 }, { id: 456 }, { id: 4636 }];
        const userWatchListCollection: IUserWatchList[] = [{ id: 123 }];
        expectedResult = service.addUserWatchListToCollectionIfMissing(userWatchListCollection, ...userWatchListArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const userWatchList: IUserWatchList = { id: 123 };
        const userWatchList2: IUserWatchList = { id: 456 };
        expectedResult = service.addUserWatchListToCollectionIfMissing([], userWatchList, userWatchList2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(userWatchList);
        expect(expectedResult).toContain(userWatchList2);
      });

      it('should accept null and undefined values', () => {
        const userWatchList: IUserWatchList = { id: 123 };
        expectedResult = service.addUserWatchListToCollectionIfMissing([], null, userWatchList, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(userWatchList);
      });

      it('should return initial array if no UserWatchList is added', () => {
        const userWatchListCollection: IUserWatchList[] = [{ id: 123 }];
        expectedResult = service.addUserWatchListToCollectionIfMissing(userWatchListCollection, undefined, null);
        expect(expectedResult).toEqual(userWatchListCollection);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
