package at.ac.uibk.gitsearch.repository.search;

import static org.hamcrest.Matchers.contains;
import static org.hamcrest.Matchers.hasProperty;
import static org.hamcrest.Matchers.is;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.domain.util.TreeNode;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestConfiguration;
import at.ac.uibk.gitsearch.service.dto.AutoCompleteEntry;
import java.io.IOException;
import java.text.ParseException;
import java.time.Instant;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import javax.ws.rs.NotFoundException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.codeability.sharing.plugins.api.search.SearchResultDTO;
import org.codeability.sharing.plugins.api.search.SearchResultsDTO;
import org.codeability.sharing.plugins.api.search.util.ExerciseId;
import org.elasticsearch.node.NodeValidationException;
import org.hamcrest.MatcherAssert;
import org.junit.Assert;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest(classes = GitsearchApp.class)
public class MetaDataRepositoryIT {

    private static final Logger LOGGER = LogManager.getLogger(MetaDataRepositoryIT.class);

    @Autowired
    ElasticSearchTestConfiguration elasticSearchTestConfiguration;

    static ElasticSearchTestConfiguration staticElasticSearchTestConfiguration;

    @BeforeEach // must be started as BeforeEach, in order to autowire the ElasticSearchTestConfiguration
    @Timeout(value = 2, unit = TimeUnit.MINUTES)
    public void setUpESServer() throws IOException, NodeValidationException {
        if (staticElasticSearchTestConfiguration == null) {
            elasticSearchTestConfiguration.startTestNode();
            staticElasticSearchTestConfiguration = elasticSearchTestConfiguration;
        }
    }

    @AfterAll
    public static void shutDownESServer() {
        if (staticElasticSearchTestConfiguration != null) {
            staticElasticSearchTestConfiguration.stopTestContainer();
            staticElasticSearchTestConfiguration = null;
        }
    }

    @BeforeEach
    public void setUpIndex() {}

    @Autowired
    private MetaDataRepository metaDataRepository;

    @Test
    void testKeywordAutocompletion() throws IOException {
        final List<AutoCompleteEntry> keywordsAutoComplete = metaDataRepository.getKeywordsAutoComplete("Jav", 10);
        MatcherAssert.assertThat(keywordsAutoComplete, contains(hasProperty("target", is("Java"))));
    }

    @Test
    void testCreatorAutocompletion() throws IOException {
        final List<AutoCompleteEntry> creatorAutoComplete = metaDataRepository.getCreatorAutoComplete("Pod", 10);
        MatcherAssert.assertThat(creatorAutoComplete, contains(hasProperty("target", is("Stefan Podlipnig"))));
        final List<AutoCompleteEntry> creatorAutoComplete2 = metaDataRepository.getCreatorAutoComplete("Po", 10);
        MatcherAssert.assertThat(creatorAutoComplete2, contains(hasProperty("target", is("Stefan Podlipnig"))));
    }

    @Test
    void testContributorAutocompletion() throws IOException {
        final List<AutoCompleteEntry> contributorAutoComplete = metaDataRepository.getContributorAutoComplete("Bast", 10);
        LOGGER.debug(contributorAutoComplete.toString());
        MatcherAssert.assertThat(contributorAutoComplete, contains(hasProperty("target", is("Daniel Bastta"))));
    }

    @Test
    void testProgrammingLanguageAutocompletion() throws IOException {
        final List<AutoCompleteEntry> plAutoComplete = metaDataRepository.getProgrammingLanguageAutoComplete("Ja", 10);
        MatcherAssert.assertThat(plAutoComplete, contains(hasProperty("target", is("JAVA"))));
    }

    @Test
    void getByExerciseId() throws IOException {
        // This test assumes that the test entries are public
        for (int i : new int[] { 1 }) {
            final String exerciseId = "[" + i + "]";
            final SearchResultDTO exerciseById = metaDataRepository.getExerciseById(exerciseId, Optional.empty());
            assertNotNull("should be found:", exerciseById);
            assertEquals("Id should be the same", exerciseId, exerciseById.getExerciseId());
        }
    }

    @Test
    @Timeout(value = 5, unit = TimeUnit.MINUTES)
    void getByExerciseIdsSince() throws IOException {
        // This test assumes that the test entries are public
        for (int i : new int[] { 1 }) {
            final String exerciseId = "[" + i + "]";
            final Instant farAgo = Instant.ofEpochMilli(0L);
            final SearchResultsDTO exerciseByIds = metaDataRepository.getExercisesById(
                Collections.singleton(exerciseId).stream(),
                Optional.empty(),
                0,
                100,
                farAgo
            );
            assertNotNull("should be found:", exerciseByIds);
            assertTrue(exerciseByIds.getHitCount() > 0); // at least one hit

            exerciseByIds
                .getSearchResult()
                .forEach(exerciseById -> assertEquals("Id should be the same", exerciseId, exerciseById.getExerciseId()));
        }
    }

    @Test
    @Timeout(value = 5, unit = TimeUnit.MINUTES)
    void getByExerciseIds() throws IOException {
        // This test assumes that the test entries are public
        for (int i : new int[] { 1 }) {
            final String exerciseId = "[" + i + "]";
            final SearchResultsDTO exerciseByIds = metaDataRepository.getExercisesById(
                Collections.singleton(exerciseId).stream(),
                Optional.empty(),
                0,
                100
            );
            assertNotNull("should be found:", exerciseByIds);
            assertTrue(exerciseByIds.getHitCount() > 0); // at least one hit

            exerciseByIds
                .getSearchResult()
                .forEach(exerciseById -> assertEquals("Id should be the same", exerciseId, exerciseById.getExerciseId()));
        }
    }

    @Test
    @Timeout(value = 5, unit = TimeUnit.MINUTES)
    void getByExerciseIdsForFuture() throws IOException {
        // This test assumes that the test entries are public
        for (int i : new int[] { 1 }) {
            final String exerciseId = "[" + i + "]";
            final Instant now = Instant.now();
            final SearchResultsDTO exerciseByIds = metaDataRepository.getExercisesById(
                Collections.singleton(exerciseId).stream(),
                Optional.empty(),
                0,
                100,
                now
            );
            assertNotNull("should be found:", exerciseByIds);
            assertEquals(0, exerciseByIds.getHitCount()); // no hit newer than now
        }
    }

    @Test
    void getByEmail() throws IOException {
        // This test assumes that the test entries are public

        final SearchResultsDTO exercisesByEmail = metaDataRepository.searchByEmail("c703257@uibk.ac.at", 100, Optional.empty());

        assertEquals("This may change if testdata changes", 3, exercisesByEmail.getHitCount());
    }

    @Test
    void getByExerciseIdNegative() throws IOException {
        // This test assumes that the test entries have consecutive ids
        String exerciseId = "999999"; // should not exist
        Assertions.assertThrows(
            NotFoundException.class,
            () -> {
                metaDataRepository.getExerciseById(exerciseId, Optional.empty());
            },
            "Exercise for id " + exerciseId + " should not exist:"
        );
    }

    @Test
    @Timeout(value = 5, unit = TimeUnit.MINUTES) // just for debugging
    void getExerciseCollectionTree() throws IOException, ParseException {
        String exerciseId = "[274]"; // should not exist
        ExerciseId id = ExerciseId.fromString(exerciseId);

        final TreeNode<SearchResultDTO> exerciseCollectionTree = metaDataRepository.getExerciseCollectionTree(id, Optional.empty());

        Assert.assertNotNull(exerciseCollectionTree);

        Assert.assertEquals("This test may fail if test data changes (1)", 2, exerciseCollectionTree.getChildren().size());
        // either first or second child contains sub-children
        Assert.assertTrue(
            "This test may fail if test data changes (2)",
            exerciseCollectionTree.getChildren().get(0).getChildren().size() == 2 ||
            exerciseCollectionTree.getChildren().get(1).getChildren().size() == 2
        );
    }
}
